import logging
import os.path as osp

import kkpyutil as util

from wpe.pathman import PathMan


class DeployTarget:
    def __init__(self, path_man: PathMan):
        self.pathMan = path_man
        self.deployTargetConfigPath = osp.join(self.pathMan.configDir, '.deploy_target.json')
        self.deployTargetConfig = {}
        self.load()

    def items(self):
        return self.deployTargetConfig.items()

    def load(self):
        if osp.isfile(self.deployTargetConfigPath):
            self.deployTargetConfig = util.load_json(self.deployTargetConfigPath)

    def create(self):
        while True:
            name = input('Target name: ')
            if name and name not in self.deployTargetConfig.keys():
                break
            logging.warning(f'Target exists: {name}')

        engine_selections = {'ue', 'unity', 'other'}
        while True:
            engine = input('Game engine type(ue/unity/other): ')
            if engine in engine_selections:
                break
            logging.warning(f'Invalid engine: {engine}')

        while True:
            root = input('Game project root: ')
            if osp.isdir(root):
                break
            logging.warning(f'Directory not exists: {root}')

        self.deployTargetConfig[name] = {
            'engine': engine,
            'root': root
        }
        logging.info(f'New target created: {root}')

    def list(self):
        if not self.deployTargetConfig:
            logging.info('There is no deploy target.')

        for name, config in self.deployTargetConfig.items():
            print(name)
            engine = config["engine"]
            print(f'\t{engine=}')
            root = config["root"]
            print(f'\t{root=}\n')

    def delete(self, target_name: str):
        if target_name in self.deployTargetConfig.keys():
            self.deployTargetConfig.pop(target_name)
            logging.info(f'Deleted target: {target_name}')

    def save(self):
        util.save_json(self.deployTargetConfigPath, self.deployTargetConfig)
        self._lazy_create_config_gitignore()

    def _lazy_create_config_gitignore(self):
        if osp.isfile(ignore_file := osp.join(self.pathMan.configDir, '.gitignore')):
            return

        logging.info(f'Creating .gitignore for config directory: {ignore_file}')
        util.save_text(ignore_file, '.deploy_target.json')



import os
import os.path as osp
import re

import kkpyutil as util


class PathMan:
    def __init__(self, cwd=None):
        self.root = cwd or os.getcwd()
        self.premakePluginLua = self.find_premake_plugin_lua_in_ancestor()
        self.pluginName = self.parse_plugin_name()
        self.configDir = osp.join(self.root, '.wpe')

    def find_premake_plugin_lua_in_ancestor(self):
        premake_script_filename = 'PremakePlugin.lua'
        while self.root:
            lua = osp.join(self.root, premake_script_filename)
            if osp.isfile(lua):
                return lua
            parent = osp.dirname(self.root)
            if parent == self.root:
                break
            self.root = parent
        raise FileNotFoundError(f'Can not find PremakePlugin.lua in ancestor directories. cwd: {os.getcwd()}')

    def parse_plugin_name(self):
        lines = util.load_lines(self.premakePluginLua, rmlineend=True)
        name_define_pattern = r'Plugin.name = ".*"'
        for line in lines:
            if matched := re.match(name_define_pattern, line):
                return matched.group().split('"')[1]




import os.path as osp
from distutils.dir_util import copy_tree
from distutils.file_util import copy_file
from pathlib import Path


__all__ = [
    'overwrite_copy',
    'path_is_under'
]


def overwrite_copy(src: str, dst: str):
    if osp.isfile(src):
        copy_file(src, dst)

    if osp.isdir(src):
        copy_tree(src, dst)


def path_is_under(child: str, parent: str) -> bool:
    return Path(parent) in Path(child).parents





import logging
import os
import os.path as osp
import platform
import subprocess
from glob import glob

# 3rd party
import kkpyutil as util

# project
from wpe.util import *
from wpe.pathman import PathMan
from wpe.deploy_target import DeployTarget


class Worker:
    def __init__(self, args, path_man=None):
        self.args = args
        self.wwiseRoot: str = os.getenv('WWISEROOT')
        self.wwiseSDKRoot: str = os.getenv('WWISESDK')
        self.wpScript = osp.join(self.wwiseRoot, 'Scripts/Build/Plugins/wp.py')

        self.pathMan = path_man or PathMan()
        self.deployTarget = DeployTarget(path_man=self.pathMan)

        self.terminatedWwise = False

    @staticmethod
    def get_platform_worker(args):
        system = platform.system()
        if system == 'Windows':
            return WindowsWorker(args)
        raise NotImplementedError(f'Not implemented for this platform: {system}')

    def main(self):
        if self.args.newDeployTarget:
            self.deployTarget.create()
        if self.args.listDeployTargets:
            self.deployTarget.list()
        if self.args.deleteDeployTarget:
            self.deployTarget.delete(self.args.deleteDeployTarget)

        if modified_target := (self.args.newDeployTarget or
                               self.args.listDeployTargets or
                               self.args.deleteDeployTarget):
            self.deployTarget.save()
            return

        self._validate_env()

        if self.args.premake:
            return self.premake()

        if self.args.build:
            return self.build()

    def premake(self):
        logging.info('Premake project')
        util.run_cmd(
            [
                'python',
                self.wpScript,
                'premake',
                'Authoring'
            ],
            cwd=self.pathMan.root
        )

    def build(self):
        self._terminate_wwise()
        self._build()
        self._copy_authoring_plugin()
        self._apply_deploy_targets()
        self._reopen_wwise()

    def _validate_env(self):
        if self.wwiseRoot is None:
            raise EnvironmentError(f'Unknown env variable: WWISEROOT\n  - Try setting environment variables in Wwise '
                                   f'Launcher')

        if self.wwiseSDKRoot is None:
            raise EnvironmentError(f'Unknown env variable: WWISESDK\n  - Try setting environment variables in Wwise '
                                   f'Launcher')

        if not osp.isfile(self.wpScript):
            raise FileNotFoundError(f'"{self.wpScript}" not found.')

    def _build(self):
        logging.info('Build authoring plugin')
        util.run_cmd(
            [
                'python',
                self.wpScript,
                'build',
                'Authoring',
                '-c',
                self.args.configuration,
                '-x',
                'x64',
                '-t',
                'vc160'
            ],
            cwd=self.pathMan.root
        )

    def _terminate_wwise(self):
        raise NotImplementedError('subclass it')

    def _copy_authoring_plugin(self):
        raise NotImplementedError('subclass it')

    def _apply_deploy_targets(self):
        raise NotImplementedError('subclass it')

    def _reopen_wwise(self):
        raise NotImplementedError('subclass it')


class WindowsWorker(Worker):
    def __init__(self, args):
        super().__init__(args)
        self.sharedPluginFiles = None

    def __lazy_query_shared_plugin_files(self) -> list:
        if self.sharedPluginFiles is not None:
            return self.sharedPluginFiles

        if self.args.configuration == 'Debug':
            # Use authoring plugin in debug mode for assert hook
            build_output_dir = osp.join(self.wwiseRoot, f'Authoring/x64/{self.args.configuration}/bin/Plugins')
        else:
            build_output_dir = osp.join(self.wwiseSDKRoot, f'x64_vc160/{self.args.configuration}/bin')

        self.sharedPluginFiles = list(filter(lambda x: not str(x).endswith('.xml'),
                                             glob(osp.join(build_output_dir, f'{self.pathMan.pluginName}.*'))))
        return self.sharedPluginFiles

    def _build(self):
        super()._build()
        logging.info('Build shared plugin')
        util.run_cmd(
            [
                'python',
                self.wpScript,
                'build',
                'Windows_vc160',
                '-c',
                self.args.configuration,
                '-x',
                'x64'
            ],
            cwd=self.pathMan.root
        )

    def _terminate_wwise(self):
        if self.args.forceCopyFile:
            cmd = ['taskkill', '/IM', 'wwise.exe', '/F']
            try:
                util.run_cmd(cmd)
                self.terminatedWwise = True
            except subprocess.CalledProcessError:
                pass

    def _copy_authoring_plugin(self):
        if self.args.configuration == 'Release':
            return

        build_output_dir = osp.join(self.wwiseRoot, f'Authoring/x64/{self.args.configuration}/bin/Plugins')
        src_files = glob(osp.join(build_output_dir, f'{self.pathMan.pluginName}.*'))
        dst_dir = osp.join(self.wwiseRoot, f'Authoring/x64/Release/bin/Plugins')
        logging.info(f'Copy authoring plugin "{src_files}", from "{build_output_dir}" to "{dst_dir}"')
        for src in src_files:
            dst = osp.join(dst_dir, osp.basename(src))
            overwrite_copy(src, dst)

    def _apply_deploy_targets(self):
        plugin_files = self.__lazy_query_shared_plugin_files()

        for name, config in self.deployTarget.items():
            deploy_path = None
            if (engine := config['engine']) == 'ue':
                deploy_path = osp.join(f'{config["root"]}/Plugins/Wwise/ThirdParty/x64_vc160/Profile/bin')
            elif engine == 'unity':
                logging.warning('Unity is currently unsupported, skipped.')
            elif engine == 'other':
                deploy_path = osp.join(f'{config["root"]}')
            else:
                logging.warning(f'Unsupported engine type: {engine}, skipped.')

            if deploy_path is not None:
                logging.info(f'Apply deploy target: {name}')
                for output in plugin_files:
                    logging.info(
                        f'Copy shared plugin "{output}" to "{deploy_path}"')
                    dst = osp.join(deploy_path, osp.basename(output))
                    overwrite_copy(output, dst)

    def _reopen_wwise(self):
        if self.args.forceCopyFile and self.terminatedWwise:
            wwise_exe = osp.join(self.wwiseRoot, 'Authoring/x64/Release/bin/Wwise.exe')
            util.run_daemon([wwise_exe])



import argparse

from wpe.core import Worker


def main():
    parser = argparse.ArgumentParser(
        prog='Plugin dev ci build tool',
        add_help=True,
        epilog='Wrapper of `wp.py`. Easy to premake, build, and deploy wwise plugins.',
        formatter_class=argparse.RawTextHelpFormatter
    )

    parser.add_argument(
        '-p',
        '--premake',
        action='store_true',
        dest='premake',
        default=False,
        required=False,
        help='Premake authoring project.'
    )

    parser.add_argument(
        '-b',
        '--build',
        action='store_true',
        dest='build',
        default=False,
        required=False,
        help='Build project.'
    )

    parser.add_argument(
        '-c',
        '--configuration',
        action='store',
        choices=('Debug', 'Profile', 'Release'),
        dest='configuration',
        default='Debug',
        required=False,
        help='Configuration to build (Debug, Release, Profile). Default value is Debug.'
    )
    parser.add_argument(
        '-f',
        '--force-copy-file',
        action='store_true',
        dest='forceCopyFile',
        required=False,
        default=False,
        help='Terminate Wwise process and copy plugin files, then reopen Wwise.'
    )
    parser.add_argument(
        '-n',
        '--new-deploy-target',
        action='store_true',
        dest='newDeployTarget',
        required=False,
        default=False,
        help='Create a new deploy target with interactive commandline.'
    )
    parser.add_argument(
        '-l',
        '--list-deploy-targets',
        action='store_true',
        dest='listDeployTargets',
        required=False,
        default=False,
        help='List all deploy targets.'
    )
    parser.add_argument(
        '-d',
        '--delete-deploy-target',
        action='store',
        dest='deleteDeployTarget',
        required=False,
        default='',
        help='Delete deploy targets by name.'
    )

    parsed_args = parser.parse_args()
    worker = Worker.get_platform_worker(parsed_args)
    worker.main()


if __name__ == '__main__':
    main()
